const env = require("./config/env");

const express = require("express");
const app = express();
app.use(express.json());

// Config AWS
const AWS = require("aws-sdk");
AWS.config.update({
  apiVersion: env.API_VERSION,
  accessKeyId: env.IAM.ACCESS_KEY,
  secretAccessKey: env.IAM.SECRET,
  region: env.REGION,
}); // for simplicity. In prod, use loadConfigFromFile, or env variables

const sqs = new AWS.SQS();

// Routes

// endpoint - get sqs status
// http://localhost:6001/status
app.get("/status", (req, res) => {
  res.status(200).json({
    status: "ok",
    data: sqs,
  });
});

// endpoint - send message to queue
// http://localhost:6001/send
app.post("/send", (req, res) => {
  let student = {
    id: req.body["id"],
    name: req.body["name"],
    email: req.body["email"],
    phone: req.body["phone"],
  };

  let params = {
    MessageBody: JSON.stringify(student),
    QueueUrl: env.QUEUE_ENDPOINT,
  };

  sqs.sendMessage(params, (err, data) => {
    if (err) {
      res.status(500).json({
        status: "500",
        err: err,
      });
    } else {
      res.status(202).json({
        status: "accepted",
        messageId: data.MessageId,
        data: "Message sent to queue",
      });
    }
  });
});

// endpoint - receive message from queue
// http://localhost:6001/receive
app.get("/receive", (req, res) => {
  let params = {
    QueueUrl: env.QUEUE_ENDPOINT,
    VisibilityTimeout: 600, // 10 min wait time for anyone else to process
  };

  sqs.receiveMessage(params, (err, data) => {
    if (err) {
      res.status(500).json({
        status: "500",
        err: err,
      });
    } else {
      if (!data.Messages) {
        res.status(200).json({
          status: "ok",
          messageId: null,
          data: "Nothing to process",
        });
      } else {
        res.status(200).json({
          status: "ok",
          messageId: data.Messages[0].MessageId,
          // receiptHandleId: data.Messages[0].ReceiptHandle,
          data: JSON.parse(data.Messages[0].Body),
        });
      }
    }
  });
});

// endpoint - purge the entire queue
// http://localhost:6001/purge
app.get("/purge", (req, res) => {
  let param = {
    QueueUrl: env.QUEUE_ENDPOINT,
  };

  sqs.purgeQueue(param, (err, data) => {
    if (err) {
      res.status(500).json({
        status: "500",
        err: err,
      });
    } else {
      res.status(200).json({
        status: "ok",
        data: "Queue purged",
      });
    }
  });
});

// endpoint - delete a message
// http://localhost:6001/delete
app.delete("/delete", (req, res) => {
  // todo - to be implemented.
  res.status(501).json({
    err: "Not implemented",
  });
});

// start server
const port = 6001;
app.listen(port, () => {
  console.log("SQS example app listening at http://localhost:%s", port);
});
